@@ -59,14 +59,26 @@ const char *_snd_module_pcm_dmix = "";
 static int shm_sum_create_or_connect(snd_pcm_direct_t *dmix)
 {
 	static int shm_sum_discard(snd_pcm_direct_t *dmix);
+	struct shmid_ds buf;
+	int tmpid, err;
 	size_t size;
 
 	size = dmix->shmptr->s.channels *
 	       dmix->shmptr->s.buffer_size *
 	       sizeof(signed int);	
+retryshm:
 	dmix->u.dmix.shmid_sum = shmget(dmix->ipc_key + 1, size, IPC_CREAT | 0666);
-	if (dmix->u.dmix.shmid_sum < 0)
-		return -errno;
+	err = -errno;
+	if (dmix->u.dmix.shmid_sum < 0){
+		if (errno == EINVAL)
+		if ((tmpid = shmget(dmix->ipc_key + 1, 0, 0666)) != -1)
+		if (!shmctl(tmpid, IPC_STAT, &buf))
+	    	if (!buf.shm_nattch) 
+		/* no users so destroy the segment */
+		if (!shmctl(tmpid, IPC_RMID, NULL))
+		    goto retryshm;
+		return err;
+	}
 	dmix->u.dmix.sum_buffer = shmat(dmix->u.dmix.shmid_sum, 0, 0);
 	if (dmix->u.dmix.sum_buffer == (void *) -1) {
 		shm_sum_discard(dmix);
@@ -214,7 +226,7 @@ static void mix_areas1(unsigned int size,
 {
 	register signed int sample, old_sample;
 
-	while (size-- > 0) {
+	for (;;) {
 		sample = *src;
 		old_sample = *sum;
 		if (*dst == 0)
@@ -230,6 +242,8 @@ static void mix_areas1(unsigned int size,
 				sample = old_sample;
 			*dst = sample;
 		} while (*sum != old_sample);
+		if (!--size)
+			return;
 		((char *)src) += src_step;
 		((char *)dst) += dst_step;
 		((char *)sum) += sum_step;
@@ -244,7 +258,7 @@ static void mix_areas2(unsigned int size,
 {
 	register signed int sample, old_sample;
 
-	while (size-- > 0) {
+	for (;;) {
 		sample = *src / 256;
 		old_sample = *sum;
 		if (*dst == 0)
@@ -260,6 +274,8 @@ static void mix_areas2(unsigned int size,
 				sample = old_sample * 256;
 			*dst = sample;
 		} while (*sum != old_sample);
+		if (!--size)
+			return;
 		((char *)src) += src_step;
 		((char *)dst) += dst_step;
 		((char *)sum) += sum_step;
@@ -366,9 +382,11 @@ static void snd_pcm_dmix_sync_area(snd_pcm_t *pcm, snd_pcm_uframes_t size)
 	dmix->slave_appl_ptr %= dmix->shmptr->s.boundary;
 	while (size > 0) {
 		transfer = appl_ptr + size > pcm->buffer_size ? pcm->buffer_size - appl_ptr : size;
-		transfer = slave_appl_ptr + transfer > dmix->shmptr->s.buffer_size ? dmix->shmptr->s.buffer_size - slave_appl_ptr : transfer;
-		size -= transfer;
+		if ((transfer = slave_appl_ptr + transfer > dmix->shmptr->s.buffer_size ? dmix->shmptr->s.buffer_size - slave_appl_ptr : transfer))
 		mix_areas(dmix, src_areas, dst_areas, appl_ptr, slave_appl_ptr, transfer);
+		if (transfer >= size)
+			return;
+		size -= transfer;
 		slave_appl_ptr += transfer;
 		slave_appl_ptr %= dmix->shmptr->s.buffer_size;
 		appl_ptr += transfer;
@@ -385,6 +403,15 @@ static int snd_pcm_dmix_sync_ptr(snd_pcm_t *pcm)
 	snd_pcm_uframes_t slave_hw_ptr, old_slave_hw_ptr, avail;
 	snd_pcm_sframes_t diff;
 	
+	switch (snd_pcm_state(dmix->spcm)) {
+	case SND_PCM_STATE_DISCONNECTED:
+		dmix->state = -ENOTTY;
+		return -ENOTTY;
+	default:
+		break;
+	}
+	if (dmix->slowptr)
+		snd_pcm_hwsync(dmix->spcm);
 	old_slave_hw_ptr = dmix->slave_hw_ptr;
 	slave_hw_ptr = dmix->slave_hw_ptr = *dmix->spcm->hw.ptr;
 	diff = slave_hw_ptr - old_slave_hw_ptr;
@@ -396,7 +423,6 @@ static int snd_pcm_dmix_sync_ptr(snd_pcm_t *pcm)
 	}
 	dmix->hw_ptr += diff;
 	dmix->hw_ptr %= pcm->boundary;
-	// printf("sync ptr diff = %li\n", diff);
 	if (pcm->stop_threshold >= pcm->boundary)	/* don't care */
 		return 0;
 	if ((avail = snd_pcm_mmap_playback_avail(pcm)) >= pcm->stop_threshold) {
@@ -420,6 +446,7 @@ static int snd_pcm_dmix_sync_ptr(snd_pcm_t *pcm)
 static int snd_pcm_dmix_status(snd_pcm_t *pcm, snd_pcm_status_t * status)
 {
 	snd_pcm_direct_t *dmix = pcm->private_data;
+	snd_pcm_state_t state;
 
 	switch (dmix->state) {
 	case SNDRV_PCM_STATE_DRAINING:
@@ -430,7 +457,8 @@ static int snd_pcm_dmix_status(snd_pcm_t *pcm, snd_pcm_status_t * status)
 		break;
 	}
 	memset(status, 0, sizeof(*status));
-	status->state = dmix->state;
+	state = snd_pcm_state(dmix->spcm);
+	status->state = state == SNDRV_PCM_STATE_RUNNING ? dmix->state : state;
 	status->trigger_tstamp = dmix->trigger_tstamp;
 	status->tstamp = snd_pcm_hw_fast_tstamp(dmix->spcm);
 	status->avail = snd_pcm_mmap_playback_avail(pcm);
@@ -442,6 +470,15 @@ static int snd_pcm_dmix_status(snd_pcm_t *pcm, snd_pcm_status_t * status)
 static snd_pcm_state_t snd_pcm_dmix_state(snd_pcm_t *pcm)
 {
 	snd_pcm_direct_t *dmix = pcm->private_data;
+	switch (snd_pcm_state(dmix->spcm)) {
+	case SND_PCM_STATE_SUSPENDED:
+		return -ESTRPIPE;
+	case SND_PCM_STATE_DISCONNECTED:
+		dmix->state = -ENOTTY;
+		return -ENOTTY;
+	default:
+		break;
+	}
 	return dmix->state;
 }
 
@@ -462,6 +499,8 @@ static int snd_pcm_dmix_delay(snd_pcm_t *pcm, snd_pcm_sframes_t *delayp)
 		return 0;
 	case SNDRV_PCM_STATE_XRUN:
 		return -EPIPE;
+	case SNDRV_PCM_STATE_DISCONNECTED:
+		return -ENOTTY;
 	default:
 		return -EBADFD;
 	}
@@ -480,6 +519,8 @@ static int snd_pcm_dmix_hwsync(snd_pcm_t *pcm)
 		return 0;
 	case SNDRV_PCM_STATE_XRUN:
 		return -EPIPE;
+	case SNDRV_PCM_STATE_DISCONNECTED:
+		return -ENOTTY;
 	default:
 		return -EBADFD;
 	}
@@ -519,6 +560,7 @@ static int snd_pcm_dmix_start(snd_pcm_t *pcm)
 	if (err < 0)
 		return err;
 	dmix->state = SND_PCM_STATE_RUNNING;
+	snd_pcm_hwsync(dmix->spcm);
 	dmix->slave_appl_ptr = dmix->slave_hw_ptr = *dmix->spcm->hw.ptr;
 	avail = snd_pcm_mmap_playback_hw_avail(pcm);
 	if (avail < 0)
@@ -735,6 +777,8 @@ static snd_pcm_fast_ops_t snd_pcm_dmix_fast_ops = {
  * \param ipc_key IPC key for semaphore and shared memory
  * \param ipc_perm IPC permissions for semaphore and shared memory
  * \param params Parameters for slave
+ * \param bindings Channel bindings
+ * \param slowptr Slow but more precise pointer updates
  * \param root Configuration root
  * \param sconf Slave configuration
  * \param stream PCM Direction (stream)
@@ -748,6 +792,7 @@ int snd_pcm_dmix_open(snd_pcm_t **pcmp, const char *name,
 		      key_t ipc_key, mode_t ipc_perm,
 		      struct slave_params *params,
 		      snd_config_t *bindings,
+		      int slowptr,
 		      snd_config_t *root, snd_config_t *sconf,
 		      snd_pcm_stream_t stream, int mode)
 {
@@ -809,6 +854,8 @@ int snd_pcm_dmix_open(snd_pcm_t **pcmp, const char *name,
 	pcm->fast_ops = &snd_pcm_dmix_fast_ops;
 	pcm->private_data = dmix;
 	dmix->state = SND_PCM_STATE_OPEN;
+	dmix->slowptr = slowptr;
+	dmix->sync_ptr = snd_pcm_dmix_sync_ptr;
 
 	if (first_instance) {
 		ret = snd_pcm_open_slave(&spcm, root, sconf, stream, mode);
@@ -891,7 +938,7 @@ int snd_pcm_dmix_open(snd_pcm_t **pcmp, const char *name,
 	
 	if (dmix->channels == UINT_MAX)
 		dmix->channels = dmix->shmptr->s.channels;
-	
+
 	snd_pcm_direct_semaphore_up(dmix, DIRECT_IPC_SEM_CLIENT);
 
 	*pcmp = pcm;
@@ -1058,7 +1105,7 @@ int _snd_pcm_dmix_open(snd_pcm_t **pcmp, const char *name,
 	snd_config_iterator_t i, next;
 	snd_config_t *slave = NULL, *bindings = NULL, *sconf;
 	struct slave_params params;
-	int bsize, psize, ipc_key_add_uid = 0;
+	int bsize, psize, ipc_key_add_uid = 0, slowptr = 0;
 	key_t ipc_key = 0;
 	mode_t ipc_perm = 0600;
 	int err;
@@ -1118,6 +1165,22 @@ int _snd_pcm_dmix_open(snd_pcm_t **pcmp, const char *name,
 			bindings = n;
 			continue;
 		}
+		if (strcmp(id, "slowptr") == 0) {
+			char *tmp;
+			err = snd_config_get_ascii(n, &tmp);
+			if (err < 0) {
+				SNDERR("The field slowptr must be a boolean type");
+				return err;
+			}
+			err = snd_config_get_bool_ascii(tmp);
+			free(tmp);
+			if (err < 0) {
+				SNDERR("The field slowptr must be a boolean type");
+				return err;
+			}
+			slowptr = err;
+			continue;
+		}
 		SNDERR("Unknown field %s", id);
 		return -EINVAL;
 	}
@@ -1155,7 +1218,7 @@ int _snd_pcm_dmix_open(snd_pcm_t **pcmp, const char *name,
 	params.period_size = psize;
 	params.buffer_size = bsize;
 
-	err = snd_pcm_dmix_open(pcmp, name, ipc_key, ipc_perm, &params, bindings, root, sconf, stream, mode);
+	err = snd_pcm_dmix_open(pcmp, name, ipc_key, ipc_perm, &params, bindings, slowptr, root, sconf, stream, mode);
 	if (err < 0)
 		snd_config_delete(sconf);
 	return err;
