@@ -132,6 +132,15 @@ static int snd_pcm_dshare_sync_ptr(snd_pcm_t *pcm)
 	snd_pcm_uframes_t slave_hw_ptr, old_slave_hw_ptr, avail;
 	snd_pcm_sframes_t diff;
 	
+	switch (snd_pcm_state(dshare->spcm)) {
+	case SND_PCM_STATE_DISCONNECTED:
+		dshare->state = SNDRV_PCM_STATE_DISCONNECTED;
+		return -ENOTTY;
+	default:
+		break;
+	}
+	if (dshare->slowptr)
+		snd_pcm_hwsync(dshare->spcm);
 	old_slave_hw_ptr = dshare->slave_hw_ptr;
 	slave_hw_ptr = dshare->slave_hw_ptr = *dshare->spcm->hw.ptr;
 	diff = slave_hw_ptr - old_slave_hw_ptr;
@@ -167,6 +176,7 @@ static int snd_pcm_dshare_sync_ptr(snd_pcm_t *pcm)
 static int snd_pcm_dshare_status(snd_pcm_t *pcm, snd_pcm_status_t * status)
 {
 	snd_pcm_direct_t *dshare = pcm->private_data;
+	snd_pcm_state_t state;
 
 	switch (dshare->state) {
 	case SNDRV_PCM_STATE_DRAINING:
@@ -177,7 +187,8 @@ static int snd_pcm_dshare_status(snd_pcm_t *pcm, snd_pcm_status_t * status)
 		break;
 	}
 	memset(status, 0, sizeof(*status));
-	status->state = dshare->state;
+	state = snd_pcm_state(dshare->spcm);
+	status->state = state == SND_PCM_STATE_RUNNING ? dshare->state : state;
 	status->trigger_tstamp = dshare->trigger_tstamp;
 	status->tstamp = snd_pcm_hw_fast_tstamp(dshare->spcm);
 	status->avail = snd_pcm_mmap_playback_avail(pcm);
@@ -189,6 +200,15 @@ static int snd_pcm_dshare_status(snd_pcm_t *pcm, snd_pcm_status_t * status)
 static snd_pcm_state_t snd_pcm_dshare_state(snd_pcm_t *pcm)
 {
 	snd_pcm_direct_t *dshare = pcm->private_data;
+	switch (snd_pcm_state(dshare->spcm)) {
+	case SND_PCM_STATE_SUSPENDED:
+		return -ESTRPIPE;
+	case SND_PCM_STATE_DISCONNECTED:
+		dshare->state = SNDRV_PCM_STATE_DISCONNECTED;
+		return -ENOTTY;
+	default:
+		break;
+	}
 	return dshare->state;
 }
 
@@ -209,6 +229,8 @@ static int snd_pcm_dshare_delay(snd_pcm_t *pcm, snd_pcm_sframes_t *delayp)
 		return 0;
 	case SNDRV_PCM_STATE_XRUN:
 		return -EPIPE;
+	case SNDRV_PCM_STATE_DISCONNECTED:
+		return -ENOTTY;
 	default:
 		return -EBADFD;
 	}
@@ -227,6 +249,8 @@ static int snd_pcm_dshare_hwsync(snd_pcm_t *pcm)
 		return 0;
 	case SNDRV_PCM_STATE_XRUN:
 		return -EPIPE;
+	case SNDRV_PCM_STATE_DISCONNECTED:
+		return -ENOTTY;
 	default:
 		return -EBADFD;
 	}
@@ -266,6 +290,7 @@ static int snd_pcm_dshare_start(snd_pcm_t *pcm)
 	if (err < 0)
 		return err;
 	dshare->state = SND_PCM_STATE_RUNNING;
+	snd_pcm_hwsync(dshare->spcm);
 	dshare->slave_appl_ptr = dshare->slave_hw_ptr = *dshare->spcm->hw.ptr;
 	avail = snd_pcm_mmap_playback_hw_avail(pcm);
 	if (avail < 0)
@@ -485,6 +510,8 @@ static snd_pcm_fast_ops_t snd_pcm_dshare_fast_ops = {
  * \param ipc_key IPC key for semaphore and shared memory
  * \param ipc_mode IPC permissions for semaphore and shared memory
  * \param params Parameters for slave
+ * \param bindings Channel bindings
+ * \param slowptr Slow but more precise pointer updates
  * \param root Configuration root
  * \param sconf Slave configuration
  * \param stream PCM Direction (stream)
@@ -498,6 +525,7 @@ int snd_pcm_dshare_open(snd_pcm_t **pcmp, const char *name,
 			key_t ipc_key, mode_t ipc_perm,
 			struct slave_params *params,
 			snd_config_t *bindings,
+			int slowptr,
 			snd_config_t *root, snd_config_t *sconf,
 			snd_pcm_stream_t stream, int mode)
 {
@@ -566,6 +594,8 @@ int snd_pcm_dshare_open(snd_pcm_t **pcmp, const char *name,
 	pcm->fast_ops = &snd_pcm_dshare_fast_ops;
 	pcm->private_data = dshare;
 	dshare->state = SND_PCM_STATE_OPEN;
+	dshare->slowptr = slowptr;
+	dshare->sync_ptr = snd_pcm_dshare_sync_ptr;
 
 	if (first_instance) {
 		ret = snd_pcm_open_slave(&spcm, root, sconf, stream, mode);
@@ -747,7 +777,7 @@ int _snd_pcm_dshare_open(snd_pcm_t **pcmp, const char *name,
 	snd_config_iterator_t i, next;
 	snd_config_t *slave = NULL, *bindings = NULL, *sconf;
 	struct slave_params params;
-	int bsize, psize, ipc_key_add_uid = 0;
+	int bsize, psize, ipc_key_add_uid = 0, slowptr = 0;
 	key_t ipc_key = 0;
 	mode_t ipc_perm = 0600;
 	
@@ -808,6 +838,22 @@ int _snd_pcm_dshare_open(snd_pcm_t **pcmp, const char *name,
 			bindings = n;
 			continue;
 		}
+		if (strcmp(id, "slowptr") == 0) {
+			char *tmp;
+			err = snd_config_get_ascii(n, &tmp);
+			if (err < 0) {
+				SNDERR("The field slowptr must be a boolean type");
+				return err;
+			}
+			err = snd_config_get_bool_ascii(tmp);
+			free(tmp);
+			if (err < 0) {
+				SNDERR("The field slowptr must be a boolean type");
+				return err;
+			}
+			slowptr = err;
+			continue;
+		}
 		SNDERR("Unknown field %s", id);
 		return -EINVAL;
 	}
@@ -843,7 +889,7 @@ int _snd_pcm_dshare_open(snd_pcm_t **pcmp, const char *name,
 
 	params.period_size = psize;
 	params.buffer_size = bsize;
-	err = snd_pcm_dshare_open(pcmp, name, ipc_key, ipc_perm, &params, bindings, root, sconf, stream, mode);
+	err = snd_pcm_dshare_open(pcmp, name, ipc_key, ipc_perm, &params, bindings, slowptr, root, sconf, stream, mode);
 	if (err < 0)
 		snd_config_delete(sconf);
 	return err;
