@@ -43,6 +43,7 @@ extern const char *_snd_module_pcm_ladspa;
 extern const char *_snd_module_pcm_dmix;
 extern const char *_snd_module_pcm_dsnoop;
 extern const char *_snd_module_pcm_dshare;
+extern const char *_snd_module_pcm_asym;
 
 static const char **snd_pcm_open_objects[] = {
 	&_snd_module_pcm_adpcm,
@@ -65,7 +66,8 @@ static const char **snd_pcm_open_objects[] = {
 	&_snd_module_pcm_ladspa,
 	&_snd_module_pcm_dmix,
 	&_snd_module_pcm_dsnoop,
-	&_snd_module_pcm_dshare
+	&_snd_module_pcm_dshare,
+	&_snd_module_pcm_asym
 };
 	
 void *snd_pcm_open_symbols(void)
